==> org.simantics.scl.compiler.source_0.6.1.v20170906-123458.jar: org/simantics/scl/compiler/elaboration/expressions/EListComprehension.java <==
package org.simantics.scl.compiler.elaboration.expressions;

import org.simantics.scl.compiler.elaboration.contexts.SimplificationContext;
import org.simantics.scl.compiler.elaboration.contexts.TranslationContext;
import org.simantics.scl.compiler.elaboration.contexts.TypingContext;
import org.simantics.scl.compiler.elaboration.expressions.list.CompiledQualifier;
import org.simantics.scl.compiler.elaboration.expressions.list.ListQualifier;
import org.simantics.scl.compiler.errors.Locations;
import org.simantics.scl.compiler.types.Type;
import org.simantics.scl.compiler.types.Types;
import org.simantics.scl.compiler.types.exceptions.MatchException;

public class EListComprehension extends SimplifiableExpression {

    public Expression head;
    public ListQualifier qualifier;
    
    public EListComprehension(Expression head, ListQualifier qualifier) {
        this.head = head;
        this.qualifier = qualifier;
    }

    public EListComprehension(long loc, Expression head,
            ListQualifier qualifier) {
        super(loc);
        this.head = head;
        this.qualifier = qualifier;
    }
    
    @Override
    public Expression checkBasicType(TypingContext context, Type requiredType) {
        qualifier.checkType(context);
        Type componentType;
        try {
            componentType = Types.unifyApply(Types.LIST, requiredType);
        } catch (MatchException e) {
            context.getErrorLog().log(location, "Expected a value with type " + requiredType + " but got a list.");
            return new EError(location);
        }
        head = head.checkType(context, componentType);
        return this;
    }
    
    @Override
    protected void updateType() throws MatchException {
        setType(Types.list(head.getType()));
    }

    @Override
    public Expression simplify(SimplificationContext context) {
        context.pushLocation(location);
        try {
            CompiledQualifier cq = qualifier.compile(context);            
            Expression exp = context.mapList(
                    context.lambda(cq.pattern, head), 
                    cq.value);
            //System.out.println("simplified: " + exp);
            /*try {
                exp.validateType(context.getEnvironment());
            } catch (TypeValidationException e) {
                throw new InternalCompilerError(e);
            }*/
            exp = exp.simplify(context);
            return exp;
        } finally {
            context.popLocation();
        }
    }

    @Override
    public Expression resolve(TranslationContext context) {
        context.pushFrame();
        qualifier.resolve(context);
        head = head.resolve(context);
        context.popFrame();
        return this;
    }

    @Override
    public void setLocationDeep(long loc) {
        if(location == Locations.NO_LOCATION) {
            location = loc;
            head.setLocationDeep(loc);
            qualifier.setLocationDeep(loc);
        }
    }
    
    @Override
    public void accept(ExpressionVisitor visitor) {
        visitor.visit(this);
    }
    
    @Override
    public Expression accept(ExpressionTransformer transformer) {
        return transformer.transform(this);
    }

}

==> org.simantics.scl.compiler.source_0.6.2.v20251121-084035.jar: org/simantics/scl/compiler/elaboration/expressions/EListComprehension.java <==
package org.simantics.scl.compiler.elaboration.expressions;

import org.simantics.scl.compiler.elaboration.contexts.SimplificationContext;
import org.simantics.scl.compiler.elaboration.contexts.TranslationContext;
import org.simantics.scl.compiler.elaboration.contexts.TypingContext;
import org.simantics.scl.compiler.elaboration.expressions.list.CompiledQualifier;
import org.simantics.scl.compiler.elaboration.expressions.list.ListQualifier;
import org.simantics.scl.compiler.errors.Locations;
import org.simantics.scl.compiler.types.Type;
import org.simantics.scl.compiler.types.Types;
import org.simantics.scl.compiler.types.exceptions.MatchException;

public class EListComprehension extends SimplifiableExpression {

    public Expression head;
    public ListQualifier qualifier;
    
    public EListComprehension(Expression head, ListQualifier qualifier) {
        this.head = head;
        this.qualifier = qualifier;
    }

    public EListComprehension(long loc, Expression head,
            ListQualifier qualifier) {
        super(loc);
        this.head = head;
        this.qualifier = qualifier;
    }
    
    @Override
    public Expression checkBasicType(TypingContext context, Type requiredType) {
        qualifier.checkType(context);
        Type componentType;
        try {
            componentType = Types.unifyApply(Types.LIST, requiredType);
        } catch (MatchException e) {
            context.getErrorLog().log(location, "Expected a value with type " + requiredType + " but got a list.");
            return new EError(location);
        }
        head = head.checkType(context, componentType);
        return this;
    }
    
    @Override
    protected void updateType() throws MatchException {
        setType(Types.list(head.getType()));
    }

    @Override
    public Expression simplify(SimplificationContext context) {
        //System.out.println("--------");
        context.pushLocation(location);
        try {
            CompiledQualifier cq = qualifier.compile(context);
            /*
            System.out.println("cq.pattern = " + cq.pattern);
            System.out.println("cq.value   = " + cq.value);
            System.out.println("head       = " + head);
            */
            Expression exp = context.mapList(
                    context.lambda(cq.pattern, head), 
                    cq.value);
            //System.out.println("simplified = " + exp);
            /*try {
                exp.validateType(context.getEnvironment());
            } catch (TypeValidationException e) {
                throw new InternalCompilerError(e);
            }*/
            exp = exp.simplify(context);
            return exp;
        } finally {
            context.popLocation();
        }
    }

    @Override
    public Expression resolve(TranslationContext context) {
        context.pushFrame();
        qualifier.resolve(context);
        head = head.resolve(context);
        context.popFrame();
        return this;
    }

    @Override
    public void setLocationDeep(long loc) {
        if(location == Locations.NO_LOCATION) {
            location = loc;
            head.setLocationDeep(loc);
            qualifier.setLocationDeep(loc);
        }
    }
    
    @Override
    public void accept(ExpressionVisitor visitor) {
        visitor.visit(this);
    }
    
    @Override
    public Expression accept(ExpressionTransformer transformer) {
        return transformer.transform(this);
    }

}
